#ifndef CHROMEOS_PLATFORMS_CAMERA_LIBHDR_HDRNET_LINEAR_RGB_PIPELINE_CROS_H_
#define CHROMEOS_PLATFORMS_CAMERA_LIBHDR_HDRNET_LINEAR_RGB_PIPELINE_CROS_H_

#include <memory>
#include <string>

#if defined(GOOGLE_UNSUPPORTED_OS_LOONIX)
// clang-format off
#include "third_party/GL/gl/include/GLES3/gl3.h"
#include "third_party/GL/gl/include/GLES2/gl2ext.h"
// clang-format on
#else
// clang-format off
#include <GLES3/gl3.h>
#include <GLES2/gl2ext.h>
// clang-format on
#endif

namespace cros {

// The interface for CrOS camera service to call into the HDRnet processing and
// rendering pipeline.
class HdrNetLinearRgbPipelineCrOS {
 public:
  // Metadata needed to import a 2D texture into the HDRnet pipeline.
  struct Texture2DInfo {
    static constexpr const int kInvalidTextureId = -1;

    // The native identifier of the 2D texture as returned by glGenTextures().
    GLint id = kInvalidTextureId;
    // The GL internal format of the 2D texture.
    GLenum internal_format;
    // `width` and `height` are the texel size of the 2D texture.
    int width;
    int height;

    bool IsValid() { return id != kInvalidTextureId; }
  };

  // Configuration options for the HDRnet pipeline. For more details, see the
  // comments for hdrnet::gl::LinearRgbPipeline::Options in
  // google3/googlex/gcam/hdrnet/gl/linear_rgb_pipeline.h
  struct Options {
    // `input_width` and `input_height` are the size of the input texture.
    int input_width, input_height;
    // 'output_width and 'output_height` are the size of the output texture that
    // HDRnet renders into.
    int output_width, output_height;
    // `intermediate_format` is the GL format that will be used in the
    // intermediate texture inside the HDRnet pipeline.
    GLenum intermediate_format = GL_RGBA16F;
    // `min_hdr_ratio` and `max_hdr_ratio` are parameters needed by the HDRnet
    // pipeline.
    float min_hdr_ratio = 0.03f;
    float max_hdr_ratio = 30.0f;
  };

  // Factory mehtod to create a LinearRgbPipelineCrOS instance. `options`
  // specifies the HDRnet pipeline configuration and will be passed to the
  // underlying hdrnet::gl::LinearRgbPipeline instance. `model_dir` specifies
  // the path on device to locate the HDRnet models. If `model_dir` is empty
  // string then the built-in model is used.
  static std::unique_ptr<HdrNetLinearRgbPipelineCrOS> CreatePipeline(
      const Options& options, const std::string model_dir = "");

  // Runs the HDRnet pipeline for coefficient prediction and HDRnet rendering.
  //
  // `input` is the input "raw" texture linearized from the 8-bit or 10-bit YUV
  // buffer produced by the ISP pipeline.
  //
  // `input_coeff_prediction` is an optional texture that will be used by the
  // HDRnet pipeline for coefficient prediction if presented. If
  // `input_coeff_prediction` is invalid then HDRnet pipeline will downsample
  // from the `input` texture to create the coefficient prediction texture
  // instead.
  //
  // `output` is the output texture that the HDRnet pipeline renders the result
  // into.
  //
  // `hdr_ratio` is the HDR ratio argument to the HDRnet rendering pipeline.
  virtual bool Run(Texture2DInfo input, Texture2DInfo input_coeff_prediction,
                   Texture2DInfo output, float hdr_ratio) = 0;

  virtual ~HdrNetLinearRgbPipelineCrOS() = default;
};

}  // namespace cros

#endif  // CHROMEOS_PLATFORMS_CAMERA_LIBHDR_HDRNET_LINEAR_RGB_PIPELINE_CROS_H_
